import { AuthenticationConfig } from '@bleco/authentication';
import { Binding, Component, ControllerClass, ProviderMap, ServiceOrProviderClass } from '@loopback/core';
import { Class, Model, Repository } from '@loopback/repository';
import { RestApplication } from '@loopback/rest';
import { IAuthServiceConfig, IMfaConfig, IOtpConfig } from './types';
export declare class AuthenticationServiceComponent implements Component {
    private readonly application;
    private readonly mfaConfig;
    private readonly otpConfig;
    private readonly authConfig?;
    private readonly config?;
    providers: ProviderMap;
    bindings: Binding[];
    /**
     * An optional list of Repository classes to bind for dependency injection
     * via `app.repository()` API.
     */
    repositories?: Class<Repository<Model>>[];
    /**
     * An optional list of Model classes to bind for dependency injection
     * via `app.model()` API.
     */
    models?: Class<Model>[];
    /**
     * An array of controller classes
     */
    controllers?: ControllerClass[];
    /**
     * An array of service or provider classes
     */
    services?: ServiceOrProviderClass[];
    constructor(application: RestApplication, mfaConfig: IMfaConfig, otpConfig: IOtpConfig, authConfig?: IAuthServiceConfig | undefined, config?: AuthenticationConfig | undefined);
    /**
     * Setup ServiceSequence by default if no other sequence provided
     */
    setupSequence(): void;
    setupAuthenticationComponent(secureClient?: boolean): void;
    setupAuthorizationComponent(): void;
    setupMultiFactorAuthentication(): void;
    setupLogoutProviders(): void;
    setupMultiTenancyComponent(): void;
}
