"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.FacebookLoginController = void 0;
const tslib_1 = require("tslib");
const authentication_1 = require("@bleco/authentication");
const context_1 = require("@loopback/context");
const repository_1 = require("@loopback/repository");
const rest_1 = require("@loopback/rest");
const core_1 = require("@loopx/core");
const user_core_1 = require("@loopx/user-core");
const url_1 = require("url");
const providers_1 = require("../../providers");
const client_auth_request_dto_1 = require("./models/client-auth-request.dto");
const token_response_dto_1 = require("./models/token-response.dto");
const queryGen = (from) => {
    return (req) => {
        return {
            state: `client_id=${req[from].client_id}`,
        };
    };
};
let FacebookLoginController = class FacebookLoginController {
    constructor(authClientRepository, logger, getAuthCode) {
        this.authClientRepository = authClientRepository;
        this.logger = logger;
        this.getAuthCode = getAuthCode;
    }
    async postLoginViaFacebook(clientCreds) {
        //do nothing
    }
    async facebookCallback(code, state, response, user) {
        const clientId = new url_1.URLSearchParams(state).get('client_id');
        if (!clientId || !user) {
            throw new authentication_1.AuthenticationErrors.ClientInvalid();
        }
        const client = await this.authClientRepository.findOne({
            where: {
                clientId,
            },
        });
        if (!(client === null || client === void 0 ? void 0 : client.redirectUrl)) {
            throw new authentication_1.AuthenticationErrors.ClientInvalid();
        }
        try {
            const token = await this.getAuthCode(client, user);
            response.redirect(`${client.redirectUrl}?code=${token}`);
        }
        catch (error) {
            this.logger.error(error);
            throw new authentication_1.AuthenticationErrors.InvalidCredentials();
        }
    }
};
exports.FacebookLoginController = FacebookLoginController;
tslib_1.__decorate([
    (0, authentication_1.authenticateClient)(authentication_1.STRATEGY.CLIENT_PASSWORD),
    (0, authentication_1.authenticate)(authentication_1.STRATEGY.FACEBOOK_OAUTH2, {
        accessType: 'offline',
        authorizationURL: process.env.FACEBOOK_AUTH_URL,
        callbackURL: process.env.FACEBOOK_AUTH_CALLBACK_URL,
        clientID: process.env.FACEBOOK_AUTH_CLIENT_ID,
        clientSecret: process.env.FACEBOOK_AUTH_CLIENT_SECRET,
        tokenURL: process.env.FACEBOOK_AUTH_TOKEN_URL,
    }, queryGen('body')),
    (0, rest_1.post)('/auth/facebook', {
        responses: {
            [core_1.STATUS_CODE.OK]: {
                description: 'POST Call for Facebook based login',
                content: {
                    [core_1.CONTENT_TYPE.JSON]: {
                        schema: { [core_1.X_TS_TYPE]: token_response_dto_1.TokenResponse },
                    },
                },
            },
        },
    }),
    tslib_1.__param(0, (0, rest_1.requestBody)({
        content: {
            [core_1.CONTENT_TYPE.FORM_URLENCODED]: {
                schema: (0, rest_1.getModelSchemaRef)(client_auth_request_dto_1.ClientAuthRequest),
            },
        },
    })),
    tslib_1.__metadata("design:type", Function),
    tslib_1.__metadata("design:paramtypes", [client_auth_request_dto_1.ClientAuthRequest]),
    tslib_1.__metadata("design:returntype", Promise)
], FacebookLoginController.prototype, "postLoginViaFacebook", null);
tslib_1.__decorate([
    (0, authentication_1.authenticate)(authentication_1.STRATEGY.FACEBOOK_OAUTH2, {
        accessType: 'offline',
        authorizationURL: process.env.FACEBOOK_AUTH_URL,
        callbackURL: process.env.FACEBOOK_AUTH_CALLBACK_URL,
        clientID: process.env.FACEBOOK_AUTH_CLIENT_ID,
        clientSecret: process.env.FACEBOOK_AUTH_CLIENT_SECRET,
        tokenURL: process.env.FACEBOOK_AUTH_TOKEN_URL,
    }, queryGen('query')),
    (0, rest_1.get)('/auth/facebook-auth-redirect', {
        responses: {
            [core_1.STATUS_CODE.OK]: {
                description: 'Facebook Redirect Token Response',
                content: {
                    [core_1.CONTENT_TYPE.JSON]: {
                        schema: { [core_1.X_TS_TYPE]: token_response_dto_1.TokenResponse },
                    },
                },
            },
        },
    }),
    tslib_1.__param(0, rest_1.param.query.string('code')),
    tslib_1.__param(1, rest_1.param.query.string('state')),
    tslib_1.__param(2, (0, context_1.inject)(rest_1.RestBindings.Http.RESPONSE)),
    tslib_1.__param(3, (0, context_1.inject)(authentication_1.AuthenticationBindings.CURRENT_USER)),
    tslib_1.__metadata("design:type", Function),
    tslib_1.__metadata("design:paramtypes", [String, String, Object, Object]),
    tslib_1.__metadata("design:returntype", Promise)
], FacebookLoginController.prototype, "facebookCallback", null);
exports.FacebookLoginController = FacebookLoginController = tslib_1.__decorate([
    tslib_1.__param(0, (0, repository_1.repository)(user_core_1.AuthClientRepository)),
    tslib_1.__param(1, (0, context_1.inject)(core_1.LOGGER.LOGGER_INJECT)),
    tslib_1.__param(2, (0, context_1.inject)(providers_1.AuthCodeBindings.AUTH_CODE_GENERATOR_PROVIDER)),
    tslib_1.__metadata("design:paramtypes", [user_core_1.AuthClientRepository, Object, Function])
], FacebookLoginController);
//# sourceMappingURL=facebook-login.controller.js.map