"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AppleOauth2VerifyProvider = void 0;
const tslib_1 = require("tslib");
const authentication_1 = require("@bleco/authentication");
const context_1 = require("@loopback/context");
const repository_1 = require("@loopback/repository");
const user_core_1 = require("@loopx/user-core");
const providers_1 = require("../../../providers");
const auth_user_model_1 = require("../models/auth-user.model");
let AppleOauth2VerifyProvider = class AppleOauth2VerifyProvider {
    constructor(userRepository, userCredsRepository, signupProvider, preVerifyProvider, postVerifyProvider) {
        this.userRepository = userRepository;
        this.userCredsRepository = userCredsRepository;
        this.signupProvider = signupProvider;
        this.preVerifyProvider = preVerifyProvider;
        this.postVerifyProvider = postVerifyProvider;
    }
    value() {
        return async (accessToken, refreshToken, idToken, profile) => {
            let user = await this.userRepository.findOne({
                where: {
                    email: profile._json.email,
                },
            });
            user = await this.preVerifyProvider(accessToken, refreshToken, profile, user);
            if (!user) {
                user = await this.signupProvider(profile);
            }
            if (!user) {
                throw new authentication_1.AuthenticationErrors.InvalidCredentials();
            }
            const creds = await this.userCredsRepository.findOne({
                where: {
                    userId: user.id,
                },
            });
            if (!creds || creds.authProvider !== 'apple' || creds.authId !== profile.id) {
                throw new authentication_1.AuthenticationErrors.InvalidCredentials();
            }
            const authUser = new auth_user_model_1.AuthUser({
                ...user,
                id: user.id,
            });
            authUser.permissions = [];
            authUser.externalAuthToken = accessToken;
            authUser.externalRefreshToken = refreshToken;
            return this.postVerifyProvider(profile, authUser);
        };
    }
};
exports.AppleOauth2VerifyProvider = AppleOauth2VerifyProvider;
exports.AppleOauth2VerifyProvider = AppleOauth2VerifyProvider = tslib_1.__decorate([
    tslib_1.__param(0, (0, repository_1.repository)(user_core_1.UserRepository)),
    tslib_1.__param(1, (0, repository_1.repository)(user_core_1.UserCredentialsRepository)),
    tslib_1.__param(2, (0, context_1.inject)(providers_1.SignUpBindings.APPLE_SIGN_UP_PROVIDER)),
    tslib_1.__param(3, (0, context_1.inject)(providers_1.VerifyBindings.APPLE_PRE_VERIFY_PROVIDER)),
    tslib_1.__param(4, (0, context_1.inject)(providers_1.VerifyBindings.APPLE_POST_VERIFY_PROVIDER)),
    tslib_1.__metadata("design:paramtypes", [user_core_1.UserRepository,
        user_core_1.UserCredentialsRepository, Function, Function, Function])
], AppleOauth2VerifyProvider);
//# sourceMappingURL=apple-oauth2-verify.provider.js.map