"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SecureResourceOwnerVerifyProvider = void 0;
const tslib_1 = require("tslib");
const authentication_1 = require("@bleco/authentication");
const repository_1 = require("@loopback/repository");
const core_1 = require("@loopx/core");
const user_core_1 = require("@loopx/user-core");
const repositories_1 = require("../../../repositories");
let SecureResourceOwnerVerifyProvider = class SecureResourceOwnerVerifyProvider {
    constructor(userRepository, utRepository, authClientRepository, otpRepository) {
        this.userRepository = userRepository;
        this.utRepository = utRepository;
        this.authClientRepository = authClientRepository;
        this.otpRepository = otpRepository;
    }
    value() {
        return async (clientId, clientSecret, username, password) => {
            let user;
            try {
                user = await this.userRepository.verifyPassword(username, password);
            }
            catch (error) {
                const otp = await this.otpRepository.get(username);
                if (!otp || otp.otp !== password) {
                    throw new authentication_1.AuthenticationErrors.InvalidCredentials();
                }
                user = await this.userRepository.findOne({
                    where: { username },
                });
                if (!user) {
                    throw new authentication_1.AuthenticationErrors.InvalidCredentials();
                }
            }
            const userTenant = await this.utRepository.findOne({
                where: {
                    userId: user.id,
                    tenantId: user.defaultTenantId,
                    status: {
                        nin: [core_1.UserStatus.REJECTED, core_1.UserStatus.INACTIVE],
                    },
                },
            });
            if (!userTenant) {
                throw new core_1.AuthErrors.UserInactive();
            }
            const client = await this.authClientRepository.findOne({
                where: {
                    clientId,
                },
            });
            if (!client) {
                throw new authentication_1.AuthenticationErrors.ClientInvalid();
            }
            // if (
            //   !client ||
            //   // eslint-disable-next-line @typescript-eslint/ban-ts-comment
            //   // @ts-ignore
            //   user.authClientIds.indexOf(client.id || 0) < 0
            // ) {
            //   throw new AuthenticationErrors.ClientInvalid();
            // }
            if (client.clientType !== authentication_1.ClientType.public && (!client.clientSecret || client.clientSecret !== clientSecret)) {
                throw new authentication_1.AuthenticationErrors.ClientVerificationFailed();
            }
            return {
                client,
                user,
            };
        };
    }
};
exports.SecureResourceOwnerVerifyProvider = SecureResourceOwnerVerifyProvider;
exports.SecureResourceOwnerVerifyProvider = SecureResourceOwnerVerifyProvider = tslib_1.__decorate([
    tslib_1.__param(0, (0, repository_1.repository)(user_core_1.UserRepository)),
    tslib_1.__param(1, (0, repository_1.repository)(user_core_1.UserTenantRepository)),
    tslib_1.__param(2, (0, repository_1.repository)(user_core_1.AuthClientRepository)),
    tslib_1.__param(3, (0, repository_1.repository)(repositories_1.OtpRepository)),
    tslib_1.__metadata("design:paramtypes", [user_core_1.UserRepository,
        user_core_1.UserTenantRepository,
        user_core_1.AuthClientRepository,
        repositories_1.OtpRepository])
], SecureResourceOwnerVerifyProvider);
//# sourceMappingURL=secure-resource-owner-verify.provider.js.map