"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.OtpRequest = exports.OtpMethodUserKeyMap = exports.OtpSupportedMethods = void 0;
const email_1 = require("../utils/email");
const phone_1 = require("../utils/phone");
exports.OtpSupportedMethods = ['sms', 'email'];
exports.OtpMethodUserKeyMap = {
    sms: 'phone',
    email: 'email',
};
class OtpRequest {
    constructor(method, contact, user) {
        this.method = method;
        this.contact = contact;
        this.user = user;
        this.contactPropName = exports.OtpMethodUserKeyMap[this.method];
    }
    static from(contact) {
        const method = resolveMethodByContact(contact);
        if (method) {
            return new OtpRequest(method, contact);
        }
    }
    static fromUser(user, contact) {
        for (const method of exports.OtpSupportedMethods) {
            const key = exports.OtpMethodUserKeyMap[method];
            if (user[key]) {
                return new OtpRequest(method, contact, user);
            }
        }
        throw new Error(`Cannot find any contact (both email and phone number) from user ${user.id}`);
    }
}
exports.OtpRequest = OtpRequest;
function resolveMethodByContact(contact) {
    if ((0, email_1.isValidEmail)(contact)) {
        return 'email';
    }
    else if ((0, phone_1.isValidPhoneNumber)(contact)) {
        return 'sms';
    }
}
//# sourceMappingURL=otp.js.map