"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MySequence = void 0;
const tslib_1 = require("tslib");
const authentication_1 = require("@bleco/authentication");
const context_1 = require("@loopback/context");
const rest_1 = require("@loopback/rest");
const core_1 = require("@loopx/core");
const multi_tenancy_1 = require("@loopx/multi-tenancy");
const berrors_1 = require("berrors");
const lodash_1 = require("lodash");
const SequenceActions = rest_1.RestBindings.SequenceActions;
const isJsonString = (str) => {
    try {
        JSON.parse(str);
    }
    catch (e) {
        return false;
    }
    return true;
};
let MySequence = class MySequence {
    constructor(findRoute, parseParams, invoke, send, reject, identifyTenant, authenticateRequest, authenticateRequestClient, logger, i18n) {
        this.findRoute = findRoute;
        this.parseParams = parseParams;
        this.invoke = invoke;
        this.send = send;
        this.reject = reject;
        this.identifyTenant = identifyTenant;
        this.authenticateRequest = authenticateRequest;
        this.authenticateRequestClient = authenticateRequestClient;
        this.logger = logger;
        this.i18n = i18n;
        this.expressMiddlewares = [];
        /**
         * Optional invoker for registered middleware in a chain.
         * To be injected via SequenceActions.INVOKE_MIDDLEWARE.
         */
        this.invokeMiddleware = () => false;
    }
    async handle(context) {
        var _a, _b;
        const requestTime = Date.now();
        try {
            const { request, response } = context;
            response.removeHeader('x-powered-by');
            this.logger.info(`Request ${request.method} ${request.url} started at ${requestTime.toString()}.
        Request Details
        Referer = ${request.headers.referer}
        User-Agent = ${request.headers['user-agent']}
        Remote Address = ${request.socket.remoteAddress}
        Remote Address (Proxy) = ${request.headers['x-forwarded-for']}`);
            if ((_a = this.expressMiddlewares) === null || _a === void 0 ? void 0 : _a.length) {
                const responseGenerated = await this.invokeMiddleware(context, this.expressMiddlewares);
                if (responseGenerated)
                    return;
            }
            const finished = await this.invokeMiddleware(context);
            if (finished)
                return;
            const route = this.findRoute(request);
            const args = await this.parseParams(request, route);
            await this.authenticateRequestClient(request);
            await this.authenticateRequest(request, response);
            await this.identifyTenant(context);
            // // TODO get permissions from user tenant role and user self
            // const isAccessAllowed: boolean = await this.checkAuthorisation(authUser?.permissions, request);
            // if (!isAccessAllowed) {
            //   throw new AuthorizationErrors.NotAllowedAccess();
            // }
            const result = await this.invoke(route, args);
            this.send(response, result);
        }
        catch (err) {
            this.logger.error(err, `Request ${context.request.method} ${context.request.url} errored out.`);
            if (process.env.NODE_ENV === 'test') {
                this.logger.error(err.stack);
            }
            const error = this._rejectErrors(err);
            if (!(error.message &&
                [authentication_1.AuthenticationErrors.TokenInvalid.message, authentication_1.AuthenticationErrors.TokenExpired.message].includes(error.message))) {
                if ((0, lodash_1.isString)(error.message)) {
                    error.message = this.i18n.__({
                        phrase: error.message,
                        locale: (_b = process.env.LOCALE) !== null && _b !== void 0 ? _b : 'en',
                    });
                }
                else {
                    error.message = error.message || 'Some error occurred. Please try again';
                }
            }
            this.reject(context, error);
        }
        finally {
            this.logger.info(`Request ${context.request.method} ${context.request.url} Completed in ${Date.now() - requestTime}ms`);
        }
    }
    /* eslint-disable @typescript-eslint/no-explicit-any */
    _rejectErrors(err) {
        var _a;
        if (!!err.table && !!err.detail) {
            if (err.code === '23505') {
                // Postgres unique index error
                return new berrors_1.BErrors.Conflict(`Unique constraint violation error ! ${err.detail}`);
            }
            else if (err.code === '23503') {
                // Postgres foreign key error
                return new berrors_1.BErrors.NotFound(`Related entity not found ! ${err.detail}`);
            }
            else if (err.code === '23502') {
                // Postgres not null constraint error
                return new berrors_1.BErrors.NotFound(`Not null constraint violation error ! ${err.detail}`);
            }
            else {
                return err;
            }
        }
        else if (err.message && isJsonString(err.message) && JSON.parse(err.message).error) {
            return JSON.parse(err.message).error;
        }
        else if (((_a = err.message) === null || _a === void 0 ? void 0 : _a.message) && isJsonString(err.message.message) && JSON.parse(err.message.message).error) {
            return JSON.parse(err.message.message).error;
        }
        else if (err.name && err.name === 'PubNubError') {
            return new berrors_1.BErrors.UnprocessableEntity(`Pubnub returned with error ! ${JSON.stringify(err)}`);
        }
        else {
            return err;
        }
    }
};
exports.MySequence = MySequence;
tslib_1.__decorate([
    (0, context_1.inject)(core_1.LxCoreBindings.EXPRESS_MIDDLEWARES, { optional: true }),
    tslib_1.__metadata("design:type", Array)
], MySequence.prototype, "expressMiddlewares", void 0);
tslib_1.__decorate([
    (0, context_1.inject)(SequenceActions.INVOKE_MIDDLEWARE, { optional: true }),
    tslib_1.__metadata("design:type", Function)
], MySequence.prototype, "invokeMiddleware", void 0);
exports.MySequence = MySequence = tslib_1.__decorate([
    tslib_1.__param(0, (0, context_1.inject)(SequenceActions.FIND_ROUTE)),
    tslib_1.__param(1, (0, context_1.inject)(SequenceActions.PARSE_PARAMS)),
    tslib_1.__param(2, (0, context_1.inject)(SequenceActions.INVOKE_METHOD)),
    tslib_1.__param(3, (0, context_1.inject)(SequenceActions.SEND)),
    tslib_1.__param(4, (0, context_1.inject)(SequenceActions.REJECT)),
    tslib_1.__param(5, (0, context_1.inject)(multi_tenancy_1.MultiTenancyBindings.ACTION)),
    tslib_1.__param(6, (0, context_1.inject)(authentication_1.AuthenticationBindings.USER_AUTH_ACTION)),
    tslib_1.__param(7, (0, context_1.inject)(authentication_1.AuthenticationBindings.CLIENT_AUTH_ACTION)),
    tslib_1.__param(8, (0, context_1.inject)(core_1.LOGGER.LOGGER_INJECT)),
    tslib_1.__param(9, (0, context_1.inject)(core_1.LxCoreBindings.i18n)),
    tslib_1.__metadata("design:paramtypes", [Function, Function, Function, Function, Function, Function, Function, Function, Object, Object])
], MySequence);
//# sourceMappingURL=sequence.js.map