"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.LoginHelperService = void 0;
const tslib_1 = require("tslib");
const authentication_1 = require("@bleco/authentication");
const context_1 = require("@loopback/context");
const core_1 = require("@loopback/core");
const repository_1 = require("@loopback/repository");
const core_2 = require("@loopx/core");
const user_core_1 = require("@loopx/user-core");
const berrors_1 = require("berrors");
let LoginHelperService = class LoginHelperService {
    constructor(userTenantRepo, logger) {
        this.userTenantRepo = userTenantRepo;
        this.logger = logger;
    }
    async verifyClientUserLogin(req, client, reqUser) {
        const currentUser = reqUser;
        if (!client) {
            this.logger.error('Auth client not found or invalid');
            throw new authentication_1.AuthenticationErrors.ClientInvalid();
        }
        if (!currentUser) {
            this.logger.error('Auth user not found or invalid');
            throw new authentication_1.AuthenticationErrors.InvalidCredentials();
        }
        const userStatus = await this.userTenantRepo.findOne({
            where: {
                userId: currentUser.id,
            },
            fields: {
                status: true,
            },
        });
        // if (!currentUser.authClientIds || currentUser.authClientIds.length === 0) {
        //   this.logger.error('No allowed auth clients found for this user in DB');
        //   throw new AuthenticationErrors.ClientUserMissing();
        //   // eslint-disable-next-line @typescript-eslint/no-explicit-any
        // }
        //
        // if ((currentUser.authClientIds as any).indexOf(client.id ?? 0) < 0) {
        //   this.logger.error('User is not allowed to access client id passed in request');
        //   throw new AuthenticationErrors.ClientInvalid();
        // }
        if ((userStatus === null || userStatus === void 0 ? void 0 : userStatus.status) === core_2.UserStatus.REGISTERED) {
            this.logger.error('User is in registered state');
            throw new berrors_1.BErrors.BadRequest('User not active yet');
        }
        return userStatus;
    }
};
exports.LoginHelperService = LoginHelperService;
exports.LoginHelperService = LoginHelperService = tslib_1.__decorate([
    (0, core_1.injectable)({ scope: core_1.BindingScope.SINGLETON }),
    tslib_1.__param(0, (0, repository_1.repository)(user_core_1.UserTenantRepository)),
    tslib_1.__param(1, (0, context_1.inject)(core_2.LOGGER.LOGGER_INJECT)),
    tslib_1.__metadata("design:paramtypes", [user_core_1.UserTenantRepository, Object])
], LoginHelperService);
//# sourceMappingURL=login-helper.service.js.map