"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.OtpService = void 0;
const tslib_1 = require("tslib");
const authentication_1 = require("@bleco/authentication");
const context_1 = require("@loopback/context");
const core_1 = require("@loopback/core");
const repository_1 = require("@loopback/repository");
const core_2 = require("@loopx/core");
const user_core_1 = require("@loopx/user-core");
const otp_1 = require("../otp/otp");
const providers_1 = require("../providers");
const repositories_1 = require("../repositories");
let OtpService = class OtpService {
    constructor(otpCacheRepo, userRepository, otpSender, logger) {
        this.otpCacheRepo = otpCacheRepo;
        this.userRepository = userRepository;
        this.otpSender = otpSender;
        this.logger = logger;
    }
    async sendOtp(request, client) {
        if (!client) {
            this.logger.error('Auth client not found or invalid');
            throw new authentication_1.AuthenticationErrors.ClientInvalid();
        }
        const req = typeof request === 'string' ? otp_1.OtpRequest.from(request) : request;
        if (!req) {
            this.logger.error('Otp applicant is invalid');
            throw new authentication_1.AuthenticationErrors.InvalidCredentials();
        }
        const res = await this.otpSender(req);
        await this.otpCacheRepo.delete(req.contact);
        await this.otpCacheRepo.set(req.contact, {
            otpSecret: res.key,
            clientId: client.clientId,
            clientSecret: client.secret,
        });
    }
};
exports.OtpService = OtpService;
exports.OtpService = OtpService = tslib_1.__decorate([
    (0, core_1.injectable)({ scope: core_1.BindingScope.SINGLETON }),
    tslib_1.__param(0, (0, repository_1.repository)(repositories_1.OtpCacheRepository)),
    tslib_1.__param(1, (0, repository_1.repository)(user_core_1.UserRepository)),
    tslib_1.__param(2, (0, context_1.inject)(providers_1.VerifyBindings.OTP_PROVIDER, { optional: true })),
    tslib_1.__param(3, (0, context_1.inject)(core_2.LOGGER.LOGGER_INJECT)),
    tslib_1.__metadata("design:paramtypes", [repositories_1.OtpCacheRepository,
        user_core_1.UserRepository, Function, Object])
], OtpService);
//# sourceMappingURL=otp.service.js.map