"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.UserAuthService = void 0;
const tslib_1 = require("tslib");
const authentication_1 = require("@bleco/authentication");
const core_1 = require("@loopback/core");
const repository_1 = require("@loopback/repository");
const core_2 = require("@loopx/core");
const user_core_1 = require("@loopx/user-core");
const bcrypt_1 = tslib_1.__importDefault(require("bcrypt"));
const berrors_1 = require("berrors");
const repositories_1 = require("../repositories");
const saltRounds = 10;
let UserAuthService = class UserAuthService {
    constructor(userRepo, getOtpRepository) {
        this.userRepo = userRepo;
        this.getOtpRepository = getOtpRepository;
    }
    async changePassword(username, newPassword, oldPassword) {
        const user = await this.userRepo.findOne({ where: { username } });
        const creds = user && (await this.userRepo.credentials(user.id).get());
        if (oldPassword) {
            // This method considers old password as OTP
            const otp = await (await this.getOtpRepository()).get(username);
            if (!otp || otp.otp !== oldPassword) {
                throw new authentication_1.AuthenticationErrors.WrongPassword();
            }
        }
        if ((creds === null || creds === void 0 ? void 0 : creds.authProvider) !== core_2.AuthProvider.INTERNAL) {
            throw new core_2.AuthErrors.PasswordCannotBeChanged();
        }
        if (!user || user.deleted || !(creds === null || creds === void 0 ? void 0 : creds.password)) {
            throw new core_2.AuthErrors.UserDoesNotExist();
        }
        else if (await bcrypt_1.default.compare(newPassword, creds.password)) {
            throw new berrors_1.BErrors.Unauthorized('Password cannot be same as previous password!');
        }
        else {
            // DO nothing
        }
        await this.userRepo.credentials(user.id).patch({
            password: await bcrypt_1.default.hash(newPassword, saltRounds),
        });
        return user;
    }
};
exports.UserAuthService = UserAuthService;
exports.UserAuthService = UserAuthService = tslib_1.__decorate([
    (0, core_1.injectable)({ scope: core_1.BindingScope.SINGLETON }),
    tslib_1.__param(0, (0, repository_1.repository)(user_core_1.UserRepository)),
    tslib_1.__param(1, repository_1.repository.getter(repositories_1.OtpRepository)),
    tslib_1.__metadata("design:paramtypes", [user_core_1.UserRepository, Function])
], UserAuthService);
//# sourceMappingURL=user-auth.service.js.map